'use strict';

var q = require('q');
var AssertionError = require('assertion-error');
var __ = require('../..');
var assertTrue = require('./asserts').assertTrue;
var assertEquals = require('./asserts').assertEquals;
var TestMatcher = require('./TestMatcher');

describe('assertThat', function () {
	it('should do nothing on success', function () {

		__.assertThat('truth', new TestMatcher());

	});

	it('should pass value to matcher', function () {
		var input = 'assertion value';
		var passedValue;

		__.assertThat(input, new TestMatcher(function (value) {
			passedValue = value;
			return true;
		}));

		assertTrue(passedValue === input, 'Received: ' + passedValue);
	});

	it('should format assertion message if matcher fails', function () {
		var thrown;

		try {
			__.assertThat('real value', new TestMatcher(function () {
				return false;
			}));
		}
		catch (e) {
			thrown = e;
		}

		assertTrue(thrown instanceof AssertionError, 'Should throw AssertionError. Threw ' + thrown);
		assertEquals(thrown.message, '\nExpected: Matcher description\n     but: was "real value"');
	});

	it('should prepend message, if available', function () {
		var thrown;

		try {
			__.assertThat('Assertion message', 'real value', new TestMatcher(function () {
				return false;
			}));
		}
		catch (e) {
			thrown = e;
		}

		assertTrue(thrown instanceof AssertionError, 'Should throw AssertionError. Threw ' + thrown);
		assertEquals(thrown.message, 'Assertion message\nExpected: Matcher description\n     but: was "real value"');
	});

	it('should pass diff representations to AssertionError', function () {
		var thrown;

		var testMatcher = new TestMatcher(function () { return false; });
		testMatcher.getExpectedForDiff = function () {
			return 'expected for diff';
		};
		testMatcher.formatActualForDiff = function (actual) {
			return 'actual for diff: ' + actual;
		};

		try {
			__.assertThat('actual value', testMatcher);
		}
		catch (e) {
			thrown = e;
		}

		assertEquals(thrown.expected, 'expected for diff');
		assertEquals(thrown.actual, 'actual for diff: actual value');
	});

	it('should throw if matcher returns a promise', function () {
		var thrown;

		try {
			__.assertThat('a value', new TestMatcher(function () {
				return q(true);
			}));
		}
		catch (e) {
			thrown = e;
		}

		assertTrue(thrown instanceof AssertionError, 'Should throw AssertionError. Threw ' + thrown);
		assertEquals(thrown.message, 'Matcher returned a promise instead of a boolean - use promiseThat for promising matchers!');
	});
});
